<?php
/*
Plugin Name: Cacbot
Plugin URI: https://cacbot.com
Description: The Comments Are a Chat Bot
Version: 23
Author: johndee
Author URI: https://generalchicken.guru
License: Copyright(C) 2024, generalchicken.guru . All rights reserved. THIS IS NOT FREE SOFTWARE.
*/

namespace Cacbot;

require_once(plugin_dir_path(__FILE__) . 'src/Cacbot/autoloader.php');

Constants::define();

//Setup WordPress filters to accommodate the plugin:
\add_filter('comment_flood_filter', '__return_false');
\add_filter('duplicate_comment_id', '__return_false');
\add_filter('wp_is_application_passwords_available', '__return_true');

AdminSettingsPage::enable();
User::enable_cacbot_role();
Conversation::enable_cacbot_conversation_cpt();

//TODO: Future development
//Functions::enableFunctionCall();

//The "Anchor Post" functionality puts the comments from one post onto another post based on the logged in user.
// Therefore enabling many private conversations on one URL:
AnchorPost::enable_comments_override();

//The main functionality in the plugin is here:
//The plugin is triggered when a user makes a comment on an active Cacbot Conversation [or Anchor Post].
Comment::enable();

//Enabled actions and backend features:
Action_Archive::enable();
PostMetaData::enable();

//These are the main features that are activated when the user is looking at a singular Cacbot Conversation or an Anchor Post:
\add_action('template_redirect', function() {
    if(\is_user_logged_in()){
        if (\is_singular()) {
            global $post;
            if (\is_singular(SLUG ."-conversation") || \get_post_meta($post->ID, ("_" . SLUG . "_anchor_post"), true)) {
                \add_action('wp_enqueue_scripts', '\Cacbot\Scripts::do_on_WordPress_action__wp_enqueue_scripts');
                \Cacbot\AnchorPost::enable_comments_override();
                \Cacbot\Comment::change_comment_form_placeholder_text($post->ID);
                ActionButtonManager::enable_custom_comment_buttons();
            }
        }
    }
});

//todo:
//ExampleConversation::enablePublishExampleConversations();

\register_activation_hook(__FILE__, '\Cacbot\ActivationHook::do_activation_hook');


require_once(plugin_dir_path(__FILE__) . 'src/update-checker/plugin-update-checker.php');
use YahnisElsts\PluginUpdateChecker\v5\PucFactory;
PucFactory::buildUpdateChecker(    'https://cacbot.com/wp-content/uploads/details.json',__FILE__,'cacbot');


function xxxcustom_register_post_meta() {
    $args = [
        'auth_callback' => '__return_true',
        'type'          => 'boolean',
        'single'        => true,
        'show_in_rest'  => true,
    ];
    \register_meta( 'cacbot-conversation', '_cacbot_system_instructions', $args );
}
