<?php

namespace Cacbot;

class Action_Archive{

    public static function enable(){
        if(isset($_GET['model'])){
            if($_GET['model'] === "archive"){
                \add_action('init', function(){
                    if(!\wp_verify_nonce($_POST['nonce'], "cacbot-action")){
                        die("Something is wrong. Bad nonce. Action_Archive.class.php(12)");
                    }
                    Action_Archive::do_archive_comments_locally($_POST['post_id']);
                    Action_Archive::do_archive_comments_on_mothership($_POST['post_id']);
                });
            }
        }
        define ("MAX_ARCHIVES_META_KEY_LABEL", "max_archives");
    }

    public static function do_archive_comments_locally($post_id) {
        $user_id = \get_current_user_id();
        $post_id = AnchorPost::filter_for_linked_post_id($post_id, $user_id);
        $max_archives = \get_post_meta($post_id, "_" . SLUG . "_" . MAX_ARCHIVES_META_KEY_LABEL, true );
        if(!empty($max_archives)){
             $post_id = intval($post_id);
             \update_post_meta($post_id, "_" . SLUG . "_" . MAX_ARCHIVES_META_KEY_LABEL, $max_archives);
        }
        // Get all comments associated with the given post_id
        $comments = \get_comments(array('post_id' => $post_id));
        // Check if there are any comments to delete
        if (!empty($comments)) {
            foreach ($comments as $comment) {
                \wp_delete_comment($comment->comment_ID, true); // 'true' forces permanent deletion
            }
        }
        self::do_increment_archive_count($post_id);
    }

    public static function do_archive_comments_on_mothership($post_id){
        $site_url = \get_site_url();
        global $Servers;
        $Cacbot_mothership_url = $Servers->mothershipURL;
        $ApplicationPassword = ApplicationPassword::get_hashed_remote_application_password(User::get_cacbot_assistant_user_id());
        $user_name = "Assistant";
        $destination_url = $Cacbot_mothership_url . "/wp-json/cacbot/v1/archive_comments";
        $response = \wp_remote_post( $destination_url, array(
                'method'      => 'POST',
                'blocking'    => true,
                'headers'     => array(),
                'body'        => array(
                    'user_name' => $user_name,
                    'cacbot_r_a_p_hashedPassword' => $ApplicationPassword,
                    'post_id'  => $post_id,
                    'url'      => $site_url,
                ),
                'cookies'     => array()
            )
        );
        if ( is_wp_error( $response ) ) {
            $error_message = $response->get_error_message();
            echo "Something went wrong: $error_message";
            die();
        }
    }

    public static function do_increment_archive_count($post_id) {
        // Retrieve the meta values for max archives and archive action count
        $max_archives = \get_post_meta($post_id, "_cacbot_max_archives", true);
        $archive_action_count = \get_post_meta($post_id, "_cacbot_archive_action_count", true);


        if (empty($max_archives)) {
            return;
        }

        // Increment the archive action count
        $archive_action_count++;
        \update_post_meta($post_id, "_cacbot_archive_action_count", $archive_action_count);

        // Check if the incremented count has reached or exceeded max archives
        if ($archive_action_count >= $max_archives) {
            // Close comments on the post
            \wp_update_post([
                'ID' => $post_id,
                'comment_status' => 'closed'
            ]);
        }
    }

}