<?php

namespace Cacbot;

class AdminSettingsPage{

    public static function enable(){
        \add_action('admin_menu', function(){
            if (!current_user_can('manage_options')) {
                return;
            }
            \add_options_page(
                SLUG_DISPLAY_NAME . " Admin Page", // Page title
                SLUG_DISPLAY_NAME, // Menu title
                'manage_options', // Capability
                SLUG . "-admin-page", // Menu slug
                'Cacbot\AdminSettingsPage::admin_page_content',
            );
        });
    }

    public static function admin_page_content()
    {
        // Check if form has been submitted
        if (isset($_POST['submit'])) {
            // Verify nonce
            check_admin_referer('cacbot_admin_page_nonce_action', 'cacbot_admin_page_nonce');

            // Update the OpenAI API Key option
            $openai_api_key = sanitize_text_field($_POST['openai-api-key']);
            update_option('openai-api-key', $openai_api_key);

            // Save the Ion Chat Protocol option
            $ion_chat_protocol = sanitize_text_field($_POST['cacbot-protocol']);
            update_option('cacbot-protocol', $ion_chat_protocol);
        }

        // Get the existing options, if any
        $existing_api_key = get_option('openai-api-key', '');
        $existing_protocol = get_option('cacbot-protocol', 'remote_node'); // Default to 'remote_node'

        ?>
        <div class="wrap">
            <h1>Cacbot Settings</h1>
            <form method="post" action="">
                <?php wp_nonce_field('cacbot_admin_page_nonce_action', 'cacbot_admin_page_nonce'); ?>
                <table class="form-table">
                    <tr>
                        <th scope="row">
                            <label for="openai-api-key">OpenAI API Key</label>
                        </th>
                        <td>
                            <input
                                    name="openai-api-key"
                                    type="password"
                                    id="openai-api-key"
                                    value="<?php echo esc_attr($existing_api_key); ?>"
                                    placeholder="Get at https://platform.openai.com/"
                                    class="regular-text"
                                    oninput="checkInput()"

                            />
                        </td>
                    </tr>
                </table>
                <p class="submit">
                    <input
                            type="submit"
                            name="submit"
                            id="submit"
                            class="button button-primary"
                            value="Save Changes"
                        <?php echo($existing_api_key === '' ? 'disabled' : ''); ?>
                    />
                </p>
                <?php //update_option("cd", 10000);
                echo(get_option("cd")); ?>
            </form>
            <script>
                function checkInput() {
                    const apiKeyInput = document.getElementById("openai-api-key");
                    const submitButton = document.getElementById("submit");
                    submitButton.disabled = apiKeyInput.value === "";
                }
            </script>
        </div><?php
    }

}