<?php

namespace Cacbot;

class ApplicationPassword
{

    /**
     * Retrieve the hashed application password by name for a given user.
     *
     * @param int $user_id User ID to retrieve the password for.
     * @param string $app_password_name (Optional) Application password name to search for. Defaults to 'Cacbot'.
     * @return string|null The application password if found, or null if not found.
     */
    public static function get_hashed_remote_application_password(int $user_id, string $app_password_name = "Cacbot")
    {
        $app_passwords = \get_user_meta($user_id, '_application_passwords', true);

        if (!is_array($app_passwords)) {
            return null; // No application passwords found
        }

        foreach ($app_passwords as $app_password) {
            if (isset($app_password['name']) && $app_password['name'] === $app_password_name) {
                return $app_password['password'];
            }
        }

        return null; // No matching application password found
    }
}