<?php

namespace Cacbot;

class Conversation
{

    public static function enable_cacbot_conversation_cpt()
    {
        \add_action('init', [self::class, 'register_cacbot_conversation_cpt']);
    }

    public static function register_cacbot_conversation_cpt()
    {
        $labels = [
            'name'                  => _x('Cacbot Conversations', 'cacbot'),
            'singular_name'         => _x('Cacbot Conversation', 'cacbot'),
            'menu_name'             => _x('Cacbot Conversations', 'cacbot'),
            'name_admin_bar'        => _x('Cacbot Conversation', 'cacbot'),
            'add_new'               => _x('Add New', 'cacbot'),
            'add_new_item'          => __('Add New Cacbot Conversation', 'cacbot'),
            'new_item'              => __('New Cacbot Conversation', 'cacbot'),
            'edit_item'             => __('Edit Cacbot Conversation', 'cacbot'),
            'view_item'             => __('View Cacbot Conversation', 'cacbot'),
            'all_items'             => __('All Cacbot Conversations', 'cacbot'),
            'search_items'          => __('Search Cacbot Conversations', 'cacbot'),
            'parent_item_colon'     => __('Parent Cacbot Conversations:', 'cacbot'),
            'not_found'             => __('No Cacbot Conversations found.', 'cacbot'),
            'not_found_in_trash'    => __('No Cacbot Conversations found in Trash.', 'cacbot')
        ];

        $args = [
            'labels'                => $labels,
            'description'           => __('A custom post type for Cacbot Conversations.', 'cacbot'),
            'public'                => true,
            'publicly_queryable'    => true,
            'show_ui'               => true,
            'show_in_menu'          => true,
            'query_var'             => true,
            'rewrite'               => ['slug' => 'cacbot-conversation'],
            'capability_type'       => 'post',
            'has_archive'           => false,
            'hierarchical'          => true,
            'menu_position'         => null,
            'rest_base'             => 'cacbot-conversations',
            'show_in_rest'          => true,
            'supports'              => ['title', 'editor', 'author', 'thumbnail', 'excerpt', 'comments', 'custom-fields', 'revisions', 'page-attributes'],
            'taxonomies'            => ['category', 'post_tag']
        ];
        \register_post_type('cacbot-conversation', $args);
    }

    /**
     * Builds a conversation title string from its components.
     *
     * @param int $remote_post_id The remote post ID.
     * @param int $user_id The user ID of the initial speaker.
     * @param string $remote_site_url The URL of the remote site.
     * @return string The constructed title string.
     */
    public static function buildCacbotConversationTitle($remote_post_id, $user_id, $remote_site_url) {
        return sprintf('%d:%d:%s', $remote_post_id, $user_id, $remote_site_url);
    }

    /**
     * Checks if the comment is on a Cacbot Conversation post or an Anchor Post.
     *
     * @param int $post_id The ID of the post.
     * @return bool
     */
    public static function is_cacbot_conversation($post_id) {
        if(\get_post_type($post_id) === SLUG . '-conversation'){
            return true;
        }
        if(\get_post_meta($post_id, "_" . SLUG . "_" . "anchor_post", true) == true){
            return true;
        }
        return false;
    }
}
