<?php
namespace Cacbot;

/**
 * Class Interlocutor
 *
 * This class handles the interlocutor logic for comments in a WordPress environment.
 * It sets the appropriate interlocutor based on the post author and user roles,
 * and controls who is allowed to comment on posts.
 */
class Interlocutor {

    public static function enable(){

    }
    /**
     * Determines the next speaker based on the last comment.
     *
     * @param Prompt $Prompt
     * @return mixed The user ID of the next speaker, or 'CLOSED'.
     */
    public static function determine_next_speaker(Prompt $Prompt) {
        // Check if comments are open for this post
        if (!comments_open($Prompt->post_id)) {
            return 'CLOSED';
        }

        // Fetch all comments for the given post ID
        $comments = \get_comments(array(
            'post_id' => $Prompt->post_id,
            'status' => 'approve',
            'orderby' => 'comment_date',
            'order' => 'DESC',
            'number' => 1 // Only get the last comment
        ));

        // If there are no comments, return 'OPEN'
        if (empty($comments)) {
            return $Prompt->interlocutor_user_id;
        }

        // Get the last comment
        $last_comment = $comments[0];
        $last_comment_user_id = intval($last_comment->user_id);


        // Determine the next speaker based on who made the last comment
        if ($last_comment_user_id === ($Prompt->interlocutor_user_id)) {
            // If the post author made the last comment, the interlocutor is the next speaker
            return $Prompt->respondent_user_id;
        } else {
            // Otherwise, the post author is the next speaker
            return $Prompt->interlocutor_user_id;
        }
    }

}