<?php

namespace Cacbot;

class PostMetaData{


    //Some meta data represents a boolean to allow a feature to activate or not
    //This method checks those. i.e. "_cacbot_action_enabled_archive"
    public static function meta_exists_and_is_true($post_id, $property){
        if ( \metadata_exists( "post", $post_id, $property ) ){
            $value = \get_post_meta ($post_id, $property, true );
            if($value){
                return true;
            }
        }
        return false;
    }


    //These are the keys used for posts
    public static function unprotect_meta_keys(){
        \add_filter('is_protected_meta', function($protected, $given_key){
            $keys = Properties::get_post_meta_keys();
            foreach($keys as $meta_key) {
                if ((PREFIX . $meta_key) === $given_key) {
                    return false; // Set to false to make this meta key non-protected
                }
            }
            return $protected; // Default behavior for other meta keys
        }, 10, 3);
    }

    public static function enable(){
        //This API is enabled on remote nodes. It uses the regular WordPress Application Password system.
        PostMetaData::unprotect_meta_keys();
        \add_action('rest_api_init', function () {
            \register_rest_route(
                "cacbot/v1",
                "meta-data",
                array(
                    'methods' => ['POST', 'GET'],
                    'callback' => function ($args) {
                        \update_post_meta($args['post_id'], $args['key'], $args['value']);
                        return '{"status": 201}';

                    },
                    'args' => array(
                        'post_id' => array(
                            'required' => true,
                            'type' => 'integer',
                            'description' => 'The post ID of the Cacbot conversation or anchor post',
                        ),
                    ),
                    'args' => array(
                        'key' => array(
                            'required' => true,
                            'type' => 'string',
                            'description' => 'The meta key of the Cacbot Conversation or anchor post, post meta data.',
                        ),
                    ),
                    'args' => array(
                        'value' => array(
                            'required' => true,
                            'description' => 'The meta value of the Cacbot Conversation or anchor post, post meta data.',
                        ),
                    ),
                    'permission_callback' => function ($args) {
                        if(\current_user_can("edit_posts")){
                            return true;
                        }else{
                            return false;
                        }
                    },
                )
            );
        });
    }
}