<?php

namespace Cacbot;

class Prompt
{

   // use CommentMeta;

    public array $choices = []; //array of Choice objects
    public array $functions_available = []; //Array of Functions objects
    public array $functions_called = [];
    public array $messages = []; //Array of Message object
    public array $Comments = []; //Array of comments

    public int    $account_owner_user_id;
    public int    $remote_account_owner_user_id;
    public string $account_owner_user_email;

    public int    $author_user_id;
    public string $author_user_email;
    public int    $author_remote_user_id;

    public int    $interlocutor_user_id;
    public string $interlocutor_user_email;
    public int    $interlocutor_remote_user_id;

    public int    $respondent_user_id;
    public string $respondent_user_email;
    public int    $respondent_remote_user_id;

    public int    $user_id;
    public string $user_email;
    public int    $remote_user_id;


    public string $status;

    public int $completion_tokens;
    public int $max_tokens;
    public int $total_tokens;
    public int $prompt_tokens;
    public int $max_replies;
    public int $max_archive_actions;
    public int $archive_action_count;

    public string $model;

    public int    $comment_id;  // ? needed
    public int    $remote_comment_id;  // ? needed
    public string $comment_content;  // ? needed

    public bool   $comments_open;
    public int    $post_id;
    public int    $remote_post_id;
    public string $post_title;
    public string $post_content;
    public array  $tags = [];

    public $open_ai_api_key;
    public $remote_open_ai_api_key;

    public string $origin_domain_url;

    //public int $interlocutor_post_id;
   // public string $wordpress_api_key;

    public string $system_instructions;

    public $response;
    public array $response_meta;
    public $response_comment_id;
    public $remote_response;

    public $functions;

    public $reply_strategy;
    // sync http response
    // async application password
    // async username/password
    // async email
    // 2 factor


    /**
     * Sets prompt properties from post meta data.
     *
     * @param int $post_id ID of the post to retrieve meta data from
     */
    public function set_Prompt_properties_from_Post_Meta(int $post_id): void
    {
        foreach (Properties::get_transferable_anchor_post_meta_keys() as $key) {
            if (!isset($this->{$key})) {
                if (\metadata_exists('post', $post_id, "_" . SLUG . "_" . $key)) {
                    $this->{$key} = \get_post_meta($post_id, "_" . SLUG . "_" . $key, true);
                }
            }
        }
    }

    public function populate_properties_from_http_parameters() {
        foreach (Properties::get_settable_http_properties() as $property) {
            if (isset($_REQUEST[$property])) {
                $this->$property = $_REQUEST[$property];
            }
        }
    }

    //these properties are stored by "ID" key locally. We need to convert the user IDs to emails for transport.
    public function populate_user_email_properties() {
        foreach (Properties::get_convertible_id_to_email_properties() as $property) {

            $id_property = $property . "_user_id";
            // Check if the ID property is set in the current object
            if (isset($this->{$id_property})) {
                $user_id = $this->{$id_property};

                // Check if the ID is an integer
                if (!is_int($user_id)) {
                    return new \WP_Error('invalid_user_id', sprintf('The user ID for %s must be an integer.', $id_property));
                }

                // Attempt to get the user object
                $user = \get_userdata($user_id);

                // Check if the user object was retrieved successfully
                if (!$user) {
                    return new \WP_Error('user_not_found', sprintf('User with ID %d not found for %s.', $user_id, $id_property));
                }

                // Set the email property in the current object
                $email_property = $property . "_user_email";
                $this->{$email_property} = $user->user_email;
            }
        }
    }

    public static function createFunctionMetadata($name, $description, $parameters)
    {
        return [
            "name" => $name,
            "description" => $description,
            "parameters" => $parameters
        ];
    }

    public function validate_and_set_Prompt_author_id_from_email()
    {

        $User = get_user_by("email", $this->author_user_email);
        if (!$User) {
            throw new \WP_Exception("Error: Author email not found");
        }
        if (!User::is_user_an_Aion($User->ID)) {
            throw new \WP_Exception("Error: Author is not an Aion");
        }
        $this->author_user_id = $User->ID;
    }

    public function send_up()
    {
        // Determine the correct API route
        $api_route = "/wp-json/cacbot/v1/prompt";
        if (isset($this->model) && $this->model === "dall-e-3") {
            $api_route = "/wp-json/cacbot/v1/dall-e-3";
        }

        // Get the mothership URL
        global $Servers;
        $Cacbot_mothership_url = $Servers->mothershipURL;



        // Send POST request
        $response = \wp_remote_post($Cacbot_mothership_url . $api_route, array(
            'method' => 'POST',
            'timeout' => 60,
            //'redirection' => 1,
            //'httpversion' => '1.1',
            //'blocking' => true,
            //'headers' => array(),
            'body' => array(
                    'prompt' => serialize($this),
                    'cacbot_r_a_p_hashedPassword' => ApplicationPassword::get_hashed_remote_application_password(User::get_cacbot_assistant_user_id()),
                    'url'      => \get_site_url(),
                    'account_owner_user_email' => $this->account_owner_user_email,
                )
            )
        );

        // Handle potential errors
        if (\is_wp_error($response)) {
            $error_message = $response->get_error_message();
            error_log("Error in send_up(): $error_message at $Cacbot_mothership_url");
            return false; // Or return a custom error response
        }
        return $response;
    }



    //These properties are settable via http and are stored as meta data on the comment

    public function xxxset_comment_meta_from_http_parameters($comment_id)
    {
        foreach ($this->settable_meta_keys as $parameter) {
            if (isset($_REQUEST[$parameter])) {
                $metaKey = "_cacbot_" . $parameter;
                update_comment_meta($comment_id, $metaKey, $_REQUEST[$parameter]);
            }
        }
    }

    public function xxxset_properties_from_HTTP_request(){
        $R = $_REQUEST;

    }

    /*
     * This function pulls data from the database at a point denoted by a specific comment on a Cacbot Conversation
     * It initializes the Prompt data structure from that comment
     */
    public function init_this_prompt($comment_id, $status)
    {
        $this->account_owner_user_id = \get_option("_cacbot_account_owner_user_id", true);
        $account_owner_user_data = \get_userdata( $this->account_owner_user_id);
        if ( ! $account_owner_user_data ) {
            die("Error: Bad account owner. Prompt.class.php (228)");
        }
        $this->account_owner_user_email = $account_owner_user_data->email;
        $this->origin_domain_url = \get_site_url();
        $this->comment_id = $comment_id;
        $Comment = \get_comment($comment_id);
        $this->comment_content = $Comment->comment_content;
        $this->post_id = $Comment->comment_post_ID;
        $this->user_id = $Comment->user_id;
        $this->user_email = $Comment->comment_author_email;
        $this->author_user_id = \get_post_field('post_author', $Comment->comment_post_ID);
        $this->author_user_email = \get_the_author_meta('user_email', $this->author_user_id);

        $this->set_Prompt_properties_from_Post_Meta($Comment->comment_post_ID);
        $this->populate_user_email_properties();
        $this->filterSystemInstructionsForInteger();
        $this->set_empty_properties_to_defaults();
        $this->set_comments();
        $this->status = $status;
        $this->populate_properties_from_http_parameters();
    }

    //if the system instructions are an integer, we fetch them from the content of the post with that post_id
    public function filterSystemInstructionsForInteger() {

        if(!isset($this->system_instructions)){
            return;
        }
        if (!is_int(intval($this->system_instructions))) {
            // Do nothing if it's not an integer, because that means the instructions are set
            return;
        }

        //It should be an integer here.
        //Get the post ID from system_instructions
        $post_id = $this->system_instructions;

        // Retrieve the post object using the post ID
        $post = get_post($post_id);

        // Check if a valid post is retrieved
        if ($post === null) {
            return; // Invalid post ID, do nothing
        }

        // Get the post content and expand shortcodes
        $post_content = apply_filters('the_content', $post->post_content);
        $expanded_content = do_shortcode($post_content);

        // Store the expanded content in system_instructions
        $this->system_instructions = $expanded_content;
    }

    //If there are properties that are empty, that cannot remain so, there are filled in here.
    public function set_empty_properties_to_defaults(){
        if(!isset($this->max_tokens)){
            $this->max_tokens = 1500;
        }

        //system_instructions
        if(!isset($this->system_instructions)){
            $this->system_instructions = SystemInstructions::getHelpfulAssistantInstructions();
        }
    }
    public function set_comments()
    {

        $this->Comments = [];

        // Get the comments for the post with ID stored in $this->post_id
        $args = array(
            'post_id' => $this->post_id,
            'status' => 'approve'
        );
        $this->Comments = get_comments($args);
    }

    public function set_messages()
    {

        // Initialize an empty array to hold the messages
        $this->messages = [];

        // Get the comments for the post with ID stored in $this->post_id
        $args = array(
            'post_id' => $this->post_id,
            'status' => 'approve'
        );
        $comments = get_comments($args);

        $post_author = get_post_field('post_author', $this->post_id);
        // Loop through each comment and add it to the messages array
        foreach ($comments as $comment) {
            if ($comment->user_id === $post_author) {
                $role = "assistant";
            } else {
                $role = "user";
            }
            $Message = [
                "role" => $role,
                "content" => $comment->comment_content
            ];
            array_push($this->messages, $Message);
        }
        array_push($this->messages, ["role" => "system", "content" => $this->system_instructions]);
        $this->messages = array_reverse($this->messages);
        $this->messages = array_values($this->messages);

    }


}
